/**
@file RESTful.h

@brief Header file containing member functions and variables

*/

#ifndef __RESTful_H__
#define __RESTful_H__

#include "mbed.h"
#include "ESP8266.h"
#include "TCPSocketConnection.h"
#include <string>
#include <stdio.h>
#include <stdlib.h>

/** 
@brief Simple library for interfacing with RESTful API using ESP8266(NodeMCU)

@brief Revision 1.0

@author Neil Hao
@date   March 2016
 *
 * Example:
 * @code
 
#include "mbed.h"
#include "PMS3003.h"
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include "RESTful.h"

Serial pc(PC_10, PC_11);
PMS3003 sensor(PA_2, PA_3, PC_5, PA_1, PA_2);
RESTful RSF(PA_9,PA_10,PA_15,9600);

int main()
{
		sensor.init();
		sensor.PowerOn();
		wait(10); //wait for sensor setup
	
		//Update Sensor Results to ThingSpeak
		//Wifi and Host information
		char* SSID = "Your SSID";
		char* Password = "Your WIFI Password";
		char* Host = "54.164.214.198";
		int Port = 80;
		char* API_GET_MSG = "GET /update?api_key=Your_APP_KEY&field1=";
		int Retry_Times = 3;
	
		//Varilables
		int error;
		char HTTPRequest_Buffer[500]="";
		char Argument_Buffer[50]="";
		char HTTPResponce[500]="";
		int HTTPResponce_Size = 500;
	
	while(1){
		sprintf(Argument_Buffer,"%d",(int)sensor.getPM2dot5());
		char* getRequest = RSF.generateGETRequest(HTTPRequest_Buffer, API_GET_MSG, Argument_Buffer);
		
		error = RSF.connectToAP(SSID, Password,Retry_Times);
		if(error == 0){
			// error code = 0 means ERROR_NONE, Connected to AP
			pc.printf("ConnectToAP Error code is: %d\r\n", error);
			pc.printf("IP address is: %s\r\n", RSF.getIPAddress());
			error = RSF.sendHTTPRequest(getRequest, HTTPResponce, HTTPResponce_Size, Host, Port,Retry_Times);
			pc.printf("the GET Request is: %s\r\n",getRequest);
			pc.printf("HTTPResponce is: %s\r\n", HTTPResponce);
			pc.printf("SendHTTPRequest Error code is %d\r\n", error);
		}else{
			//Connect to AP failed
			pc.printf("Connect to AP failed! Please check your SSDI and Password!\r\n");
		}
		
		wait(15);
	}
}
 
 * @endcode
 */
 
 enum Wifi_Error {
    ERROR_NONE_ERROR = 0,
    ERROR_HardwareInit =1,
    ERROR_APConnection =2 ,
    ERROR_HostConnection =3,
		Responce_Received = 4,
		Empty_Responce = 5,
		ERROR_Resopnce_NOT_Received = 6,
		ERROR_GET_Request_Failed = 7
} ;

class RESTful
{
private:
  // ESP826 interface
  ESP8266 * ark;
	//TCP Socket Interface
	TCPSocketConnection * gm;
  bool HardwareInit();
	bool APConnection(char* SSID, char* Password);
	bool HostConnection(char* Host, int Port);
	

public:
    /** Create a RESTful object connected to the specified pins of ESP8266
    *
    * @param tx Pin connected to TX of ESP8266
    * @param rx Pin connected to RX of ESP8266
		* @param rest Pin connected to RESET of ESP8266
		* @param baud is the baud of USART interface,the default value is 9600 bps
		* @param timeout of conncetion, the default value is 3000 ms
    * 
    */
    RESTful(PinName tx, PinName rx, PinName reset, int baud = 9600, int timeout = 3000);


    /** Initialise RESTful
    *
    */
		int connectToAP(char* SSID, char* Password, int Retry_Times);
		int sendHTTPRequest(char* HTTPRequest, char*HTTPResponce, int HTTPResponce_Size, char* Host, int Port, int Retry_Times);

		char* generateGETRequest(char* HTTPRequest, char* API_GET_MSG, char* Argument_String);
		const char* getIPAddress();

		bool GetWifiCfg(char* SSID, char* Password, int Maximum_Response_Length);
		bool WifiSetup();
};


#endif /* __RESTful_H__ */
