/**
@file PMS3003.h

@brief Header file containing member functions and variables

*/

#ifndef __PMS3003_H__
#define __PMS3003_H__

#include "mbed.h"

/** 
@brief Simple library for interfacing with PMS3003

@brief Revision 1.0

@author Neil Hao
@date   January 2016
 *
 * Example:
 * @code
 
#include "mbed.h"
#include "PMS3003.h"

//------------------------------------
// Hyperterminal configuration
// 9600 bauds, 8-bit data, no parity
//------------------------------------
Serial pc(PC_10, PC_11);

//PM2.5 Sensor
PMS3003 sensor(PA_2, PA_3, PC_5, PA_1, PA_2);

 
int main() {
	sensor.init();
	sensor.PowerOn();
	wait(10); //the sensor entering stable status requires 10s
		
	while(1){
			if(sensor.isNormal()){
				pc.printf("PM1: %d ug/m3	PM2.5: %d ug/m3	PM10: %d ug/m3 \r\n", sensor.getPM1() , sensor.getPM2dot5(), sensor.getPM10());
				wait_ms(200);
			}
		}		
}
 
 * @endcode
 */

class PMS3003
{
private:

  // USART interface
  Serial * USART;
	//Control lines
	DigitalOut * RESET;
	DigitalOut * SET;
	DigitalOut * EN;

	//Serial * pc; //For debug

	//define how many bytes will be stored in the shift register
	#define Shift_Register_Size 24
	char Shift_Register[Shift_Register_Size];

	//Results, update interval is 2.5ms
	struct {
			unsigned short CF1PM1, CF1PM2dot5, CF1PM10, PM1, PM2dot5, PM10;
	} PM_Data;
 
	bool External_EN; //ture for has external EN design, false for without external EN design
	bool is_normal; //not completing init or hardware problem could cause the is_normal be assigned to false

	
	void readData();


public:
    /** Create a PMS3003 object connected to the specified pins, PMS3003 with external power on/off design, PMS3003_EN
    *
    * @param USART_TX  Pin connected to TX
    * @param USART_RX Pin connected to RX
		* @param PMS3003_RESET Pin connected to RESET
		* @param PMS3003_SET Pin connected to SET
		* @param PMS3003_EN Pin connected to EN
    * 
    */
    PMS3003(PinName USART_TX, PinName USART_RX, PinName PMS3003_RESET, PinName PMS3003_SET, PinName PMS3003_EN);

    /** Create a PMS3003 object connected to the specified pins, PMS3003 without external power on/off design
    *
    * @param USART_TX  Pin connected to TX
    * @param USART_RX Pin connected to RX
		* @param PMS3003_RESET Pin connected to RESET
		* @param PMS3003_SET Pin connected to SET
    * 
    */
    PMS3003(PinName USART_TX, PinName USART_RX, PinName PMS3003_RESET, PinName PMS3003_SET);
    
		
    /** Initialise PMS3003
    *
    */
    void init();
    unsigned short getCF1PM1();
    unsigned short getCF1PM2dot5();
		unsigned short getCF1PM10();
    unsigned short getPM1();
    unsigned short getPM2dot5();
		unsigned short getPM10();
		void PowerOff();
		void PowerOn();
		void Reset();
		bool isNormal();
};


#endif /* __PMS3003_H__ */
