#include "mbed.h"
#include "UC1701.h"

//Create a UC1701 object connected to the specified pins, with UC1701 back light design, UC1701_BL
UC1701::UC1701(int SPI_Hz, PinName UC1701_MOSI, PinName UC1701_MISO, PinName UC1701_SCK, PinName UC1701_CS, PinName UC1701_RST, PinName UC1701_A0,PinName UC1701_BL){
	_SPI = new SPI(UC1701_MOSI, UC1701_MISO, UC1701_SCK);
	_SPI->format(8,0);
	_SPI->frequency(SPI_Hz);
	_BL = new DigitalOut(UC1701_BL);
	_BL->write(0); //Low level for disable back light
	_A0 = new DigitalOut(UC1701_A0);
	_A0->write(0); 
	_RST = new DigitalOut(UC1701_RST);
	_RST->write(1); 
	_CS = new DigitalOut(UC1701_CS);
	_CS->write(1); 
}

//Create a UC1701 object connected to the specified pins, without UC1701 back light design
UC1701::UC1701(int SPI_Hz, PinName UC1701_MOSI, PinName UC1701_MISO, PinName UC1701_SCK, PinName UC1701_CS, PinName UC1701_RST, PinName UC1701_A0){
	_SPI = new SPI(UC1701_MOSI, UC1701_MISO, UC1701_SCK);
	_SPI->format(8,0);
	_SPI->frequency(SPI_Hz);
	_A0 = new DigitalOut(UC1701_A0);
	_A0->write(0); 
	_RST = new DigitalOut(UC1701_RST);
	_RST->write(1); 
	_CS = new DigitalOut(UC1701_CS);
	_CS->write(1); 
}

// Delay mumber of MSs by using RTOS or loop
// input: ms
void UC1701::RTOS_Delay_ms(int ms){
	wait_ms(ms); //wait using loop
}

// Send command to display controller
// input: cmd - 8-bit command to send
void UC1701::UC1701_cmd(uint8_t cmd) {
	_A0->write(0); //Low level for command mode
	_SPI->write(cmd);
}

// Send double byte command to display controller
// input: cmd1 - first byte of command to send
//        cmd2 - second byte of command to send
void UC1701::UC1701_cmd_double(uint8_t cmd1, uint8_t cmd2) {
	_A0->write(0); //Low level for command mode
	_SPI->write(cmd1);
	_SPI->write(cmd2);
}

// Send data byte to display controller
// input: data - date byte to send
void UC1701::UC1701_data(uint8_t data) {
	_A0->write(1); //High level for data mode
	_SPI->write(data);
}

/*---------------------------------
// Set backlight brightness
// input: brightness - backlight brightness level (0..100); 0 means PWM off
void UC1701::UC1701_SetBacklight(uint8_t brightness) {

}
*/

// Display initialization
void UC1701::UC1701_Init(void) {
	// Reset display
	_CS->write(1); 
	_RST->write(0);
	//Delay_ms(1); // Must hold RST low at least 1ms
	RTOS_Delay_ms(1);
	_RST->write(1);
	//Delay_ms(5); // Wait at least 5ms
	RTOS_Delay_ms(5);
	_CS->write(0); 

	UC1701_cmd(0xe2); // Software display reset
	UC1701_cmd(0x2f); // Power control: Boost ON,  V.Regular ON,  V.Follower ON
	UC1701_cmd(0xa2); // Set LCD bias ratio (BR = 0)
	UC1701_cmd(0xaf); // Display enable

	UC1701_cmd_double(0xfa,0x93); // Advanced program control 0:
								  //   Temperature compensation -0.11%/C
								  //   PA wrap around enabled, CA wrap around enabled
	_CS->write(1); 
	
	//Init screen info
	scr_width        = SCR_W;
  scr_height       = SCR_H;
	scr_orientation  = scr_normal;
}

// Reset display registers (software display reset)
// Doesn't affect the display memory
void UC1701::UC1701_Reset(void) {
	_CS->write(0);
	UC1701_cmd(0xe2); // Software display reset
	_CS->write(1);
}

// Set LCD contrast
// input:
//   res_ratio: internal resistor ratio [0..7], power on value is 4
//   el_vol: electronic volume [0..63], power on value is 32
void UC1701::UC1701_Contrast(uint8_t res_ratio, uint8_t el_vol) {
	_CS->write(0);
	UC1701_cmd(0x20 | (res_ratio & 0x07));
	UC1701_cmd_double(0x81,el_vol & 0x3f);
	_CS->write(1);
}

// Set all LCD pixels on or off
// input:
//   state: ON or OFF
// note: doesn't affect the display memory
void UC1701::UC1701_SetAllPixelOn(OnOffStatus state) {
	_CS->write(0);
	UC1701_cmd(state == ON ? 0xa5 : 0xa4);
	_CS->write(1);
}

// Set inverse display pixels
// Input:
//   state: NORMAL or INVERT
void UC1701::UC1701_SetInvert(InvertStatus state) {
	_CS->write(0);
	UC1701_cmd(state == NORMAL ? 0xa6 : 0xa7);
	_CS->write(1);
}

// Toggle display on/off
// input:
//   state: ENABLED or DISABLED
// note: doesn't affect the display memory
void UC1701::UC1701_SetDisplayState(DisplayState state) {
	_CS->write(0);
	UC1701_cmd(state == ENABLED ? 0xaf : 0xae);
	_CS->write(1);
}

// Set X coordinate mapping (normal or mirrored)
// input:
//   state: NORMAL or INVERT
// note: Doesn't affect the display memory
void UC1701::UC1701_SetXDir(InvertStatus MX) {
	_CS->write(0);
	UC1701_cmd(MX == NORMAL ? 0xa0 : 0xa1);
	_CS->write(1);
}

// Set Y coordinate mapping (normal or mirrored)
// input:
//   state: NORMAL or INVERT
// note: doesn't affect the display memory
void UC1701::UC1701_SetYDir(InvertStatus MY) {
	_CS->write(0);
	UC1701_cmd(MY == NORMAL ? 0xc8 : 0xc0);
	_CS->write(1);
}

// Set display column:page according to pixel coordinates
// input:
//   X, Y - pixel coordinates
void UC1701::UC1701_SetAddr(uint8_t X, uint8_t Y) {
	_CS->write(0);
	UC1701_cmd(X & 0x0f); // Column address LSB
	UC1701_cmd((X > 4) | 0x10); // Column address MSB
	UC1701_cmd(((Y / 8) & 0x0f) | 0xb0); // Page
	_CS->write(1);
}

// Set scroll line number
// input:
//   line - start line number (0..63)
void UC1701::UC1701_SetScrollLine(uint8_t line) {
	_CS->write(0);
	UC1701_cmd(0x40 | (line & 0x3f));
	_CS->write(1);
}

// Set display orientation
// input:
//   orientation - screen orientation (scr_*)
// note: doesn't affect on current display memory
void UC1701::UC1701_Orientation(uint8_t orientation) {
	_CS->write(0);
	switch(orientation) {
	case scr_CW:
		scr_width  = SCR_H;
		scr_height = SCR_W;
		UC1701_SetXDir(NORMAL);
		UC1701_SetYDir(INVERT);
		break;
	case scr_CCW:
		scr_width  = SCR_H;
		scr_height = SCR_W;
		UC1701_SetXDir(INVERT);
		UC1701_SetYDir(NORMAL);
		break;
	case scr_180:
		scr_width  = SCR_W;
		scr_height = SCR_H;
		UC1701_SetXDir(INVERT);
		UC1701_SetYDir(INVERT);
		break;
	default:
		scr_width  = SCR_W;
		scr_height = SCR_H;
		UC1701_SetXDir(NORMAL);
		UC1701_SetYDir(NORMAL);
		break;
	}
	scr_orientation = (ScrOrientation_TypeDef)orientation;
	_CS->write(1);
}

// Send vRAM buffer content into display
void UC1701::UC1701_Flush(void) {
	uint16_t i,j;
	uint16_t offset;

	offset = 0;
	_CS->write(0);
	for (j = 0; j < 8; j++) {
		 // Column 0 address LSB
		if (scr_orientation == scr_180 || scr_orientation == scr_CCW) UC1701_cmd(0x04); else UC1701_cmd(0x00);
		UC1701_cmd(0x10); // Column 0 address MSB
		UC1701_cmd(0xb0 | j); // Page address
		_A0->write(1); // Send data
		for (i = 0; i < 128; i++){
			UC1701_data(vRAM[offset + i]);
		}
		offset += SCR_W; // Don't use multiplication
	}
	_CS->write(1);
}

// Fill vRAM memory with specified pattern
// input:
//   pattern - byte pattern to fill vRAM memory
void UC1701::UC1701_Fill(uint8_t pattern) {
	memset(vRAM,pattern,sizeof(vRAM));
}

// Set pixel in vRAM buffer
// input:
//   X,Y - coordinates of pixel
void UC1701::SetPixel(uint8_t X, uint8_t Y) {
	uint8_t XX = X;
	uint8_t YY = Y;

	if (scr_orientation == scr_CW || scr_orientation == scr_CCW) {
		XX = Y; YY = X;
	}
	vRAM[((YY >> 3) * SCR_W) + XX] |= 1 << (YY % 8);
}

// Clear pixel in vRAM buffer
// input:
//   X,Y - coordinates of pixel
void UC1701::ResetPixel(uint8_t X, uint8_t Y) {
	uint8_t XX = X;
	uint8_t YY = Y;

	if (scr_orientation == scr_CW || scr_orientation == scr_CCW) {
		XX = Y; YY = X;
	}
	vRAM[((YY >> 3) * SCR_W) + XX] &= ~(1 << (YY % 8));
}

// Invert rectangle in vRAM buffer
// input:
//   X,Y - top left coordinates of rectangle
//   W - rectangle width
//   H - rectangle height
void UC1701::InvertRect(uint8_t X, uint8_t Y, uint8_t W, uint8_t H) {
	uint8_t i,j,pX;

	for (j = 0; j < H; j++) {
		pX = X;
		for (i = 0; i < W; i++) {
		 	vRAM[((Y >> 3) * SCR_W) + pX] ^= (1 << (Y % 8));
		 	pX++;
		}
		Y++;
	}
}

// Draw horizontal line
// input:
//   X1 - left horizontal coordinate of the line
//   X2 - right horizontal coordinate of the line
//   Y - vertical coordinate of the line
//   SR - Set or reset line pixels (PSet or PReset)
// note: X2 must be greater than X1
void UC1701::HLine(uint8_t X1, uint8_t X2, uint8_t Y, PSetReset_TypeDef SR) {
	uint8_t x;

	if (SR == PSet) {
		for (x = X1; x <= X2; x++) SetPixel(x,Y);
	} else {
		for (x = X1; x <= X2; x++) ResetPixel(x,Y);
	}
}

// Draw vertical line
// input:
//   X - horizontal coordinate of the line
//   Y1 - top vertical coordinate of the line
//   Y2 - bottom vertical coordinate of the line
//   SR - Set or reset line pixels (PSet or PReset)
// note: Y2 must be greater than Y1
void UC1701::VLine(uint8_t X, uint8_t Y1, uint8_t Y2, PSetReset_TypeDef SR) {
	uint8_t y;

	if (SR == PSet) {
		for (y = Y1; y <= Y2; y++) SetPixel(X,y);
	} else {
		for (y = Y1; y <= Y2; y++) ResetPixel(X,y);
	}
}

// Draw rectangle
// input:
//   X1 - left horizontal coordinate of the line
//   X2 - right horizontal coordinate of the line
//   Y1 - top vertical coordinate of the line
//   Y2 - bottom vertical coordinate of the line
//   SR - Set or reset rectangle pixels (PSet or PReset)
// note: must be (X2 > X1) and (Y2 > Y1)
void UC1701::Rect(uint8_t X1, uint8_t Y1, uint8_t X2, uint8_t Y2, PSetReset_TypeDef SR) {
	HLine(X1,X2,Y1,SR);
	HLine(X1,X2,Y2,SR);
	VLine(X1,Y1 + 1,Y2 - 1,SR);
	VLine(X2,Y1 + 1,Y2 - 1,SR);
}

// Draw filled rectangle
// input:
//   X1 - left horizontal coordinate of the line
//   X2 - right horizontal coordinate of the line
//   Y1 - top vertical coordinate of the line
//   Y2 - bottom vertical coordinate of the line
//   SR - Set or reset rectangle pixels (PSet or PReset)
// note: must be (X2 > X1) and (Y2 > Y1)
void UC1701::FillRect(uint8_t X1, uint8_t Y1, uint8_t X2, uint8_t Y2, PSetReset_TypeDef SR) {
	uint8_t y;

	for (y = Y1; y <= Y2; y++) HLine(X1,X2,y,SR);
}

// Draw line
// input:
//   X1 - left horizontal coordinate of the line
//   X2 - right horizontal coordinate of the line
//   Y1 - top vertical coordinate of the line
//   Y2 - bottom vertical coordinate of the line
void UC1701::Line(int16_t X1, int16_t Y1, int16_t X2, int16_t Y2) {
	int16_t dX = X2-X1;
	int16_t dY = Y2-Y1;
	int16_t dXsym = (dX > 0) ? 1 : -1;
	int16_t dYsym = (dY > 0) ? 1 : -1;
	
	int16_t dX2 = dX << 1;
	int16_t dY2 = dY << 1;
	int16_t di;

	if (dX == 0) {
		if (Y2 > Y1) VLine(X1,Y1,Y2,PSet); else VLine(X1,Y2,Y1,PSet);
		return;
	}
	if (dY == 0) {
		if (X2 > X1) HLine(X1,X2,Y1,PSet); else HLine(X2,X1,Y1,PSet);
		return;
	}

	dX *= dXsym;
	dY *= dYsym;


	if (dX >= dY) {
		di = dY2 - dX;
		while (X1 != X2) {
			SetPixel(X1,Y1);
			X1 += dXsym;
			if (di < 0) {
				di += dY2;
			} else {
				di += dY2 - dX2;
				Y1 += dYsym;
			}
		}
	} else {
		di = dX2 - dY;
		while (Y1 != Y2) {
			SetPixel(X1,Y1);
			Y1 += dYsym;
			if (di < 0) {
				di += dX2;
			} else {
				di += dX2 - dY2;
				X1 += dXsym;
			}
		}
	}
	SetPixel(X1,Y1);
}

// Draw ellipse
// input:
//   X,Y - ellipse center coordinates
//   A,B - horizontal and vertical radius of the ellipse
void UC1701::Ellipse(uint16_t X, uint16_t Y, uint16_t A, uint16_t B) {
	int16_t Xc = 0, Yc = B;

	long A2 = (long)A*A, B2 = (long)B*B;
	long C1 = -(A2/4 + A % 2 + B2);
	long C2 = -(B2/4 + B % 2 + A2);
	long C3 = -(B2/4 + B % 2);
	long t = -A2 * Yc;
	long dXt = B2*Xc*2, dYt = -A2*Yc*2;
	long dXt2 = B2*2, dYt2 = A2*2;
	while (Yc >= 0 && Xc <= A) {
		SetPixel(X + Xc,Y + Yc);
		if (Xc != 0 || Yc != 0) SetPixel(X - Xc,Y - Yc);
		if (Xc != 0 && Yc != 0) {
			SetPixel(X + Xc,Y - Yc);
			SetPixel(X - Xc,Y + Yc);
		}
		if (t + Xc*B2 <= C1 || t + Yc*A2 <= C3) {
			Xc++;
			dXt += dXt2;
			t   += dXt;
		} else if (t - Yc*A2 > C2) {
			Yc--;
			dYt += dYt2;
			t   += dYt;
		} else {
			Xc++;
			Yc--;
			dXt += dXt2;
			dYt += dYt2;
			t   += dXt;
			t   += dYt;
		}
	}
}

// Draw single character
// input:
//   X,Y - character top left corner coordinates
//   Char - character to be drawn
//   Font - pointer to font
// return: character width in pixels
uint8_t UC1701::PutChar(uint8_t X, uint8_t Y, uint8_t Char, const Font_TypeDef *Font, unsigned char const* Font_Charset) {
	uint8_t chW,chH;
	uint8_t pX = X;
	uint8_t pY = Y;
	uint16_t i,j,k;
	uint8_t buffer[32];
	uint8_t tmpCh;

	chW = Font->font_Width; // Character width
	chH = Font->font_Height; // Character height
	if (Char < 32 || Char > 0x7e) Char = 0x7e;
	memcpy(buffer,&Font_Charset[(Char - 32) * Font->font_BPC],Font->font_BPC);

	if (Font->font_Scan == font_V) {
		if (chH < 8) {
			// Small font, one byte height
			for (i = 0; i < chW; i++) {
				tmpCh = buffer[i];
				for (j = 0; j < chH; j++) {
					if ((tmpCh >> j) & 0x01) SetPixel(pX,pY);
					pY++;
				}
				pY = Y;
				pX++;
			}
		} else {
			// Big font, more than one byte height
			for (k = 0; k <= (chH / 8); k++) {
				for (i = 0; i < chW; i++) {
					tmpCh = buffer[(i << 1) + k];
					for (j = 0; j < 8; j++) {
						if ((tmpCh >> (7 - j)) & 0x01) SetPixel(pX,pY);
						pY++;
						if (pY > Y + chH) break;
					}
					pX++;
				}
				pY = Y + (k << 3);
				pX = X;
			}
		}
	} else {
		for (j = 0; j < chH; j++) {
			tmpCh = buffer[j];
			for (i = 0; i < chW; i++) {
				if ((tmpCh >> i) & 0x01) SetPixel(pX,pY);
				pX++;
			}
			pX = X;
			pY++;
		}
	}

	return Font->font_Width + 1;
}

// Draw string
// input:
//   X,Y - top left coordinates of first character
//   str - pointer to zero-terminated string
//   Font - pointer to font
// return: string width in pixels
uint16_t UC1701::PutStr(uint8_t X, uint8_t Y, char *str, const Font_TypeDef *Font , unsigned char const* Font_Charset) {
	uint8_t strLen = 0;

    while (*str) {
        X += PutChar(X,Y,*str++,Font, Font_Charset);
        if (X > scr_width - Font->font_Width - 1) break;
        strLen++;
    };

    return strLen * (Font->font_Width + 1);
}

/*
// Draw string with line feed (by screen edge)
// input:
//   X,Y - top left coordinates of first character
//   str - pointer to zero-terminated string
//   Font - pointer to font
// return: string width in pixels
uint16_t UC1701::PutStrLF(uint8_t X, uint8_t Y, char *str, const Font_TypeDef *Font, unsigned char const* Font_Charset) {
	uint8_t strLen = 0;

    while (*str) {
        PutChar(X,Y,*str++,Font, Font_Charset);
        if (X < scr_width - Font->font_Width - 1) {
        	X += Font->font_Width + 1;
        } else if (Y < scr_height - Font->font_Height - 1) {
        	X = 0; Y += Font->font_Height;
        } else {
        	X = 0; Y = 0;
        }
        strLen++;
    };

    return strLen * (Font->font_Width + 1);
}

// Draw signed integer value
// input:
//   X,Y - top left coordinates of first symbol
//   num - signed integer value
//   Font - pointer to font
// return: number width in pixels
uint8_t UC1701::PutInt(uint8_t X, uint8_t Y, int32_t num, const Font_TypeDef *Font, unsigned char const* Font_Charset) {
	uint8_t str[11]; // 10 chars max for UINT32_MAX
	int8_t i = 0;
	uint8_t neg = 0;
	int8_t intLen;
	uint8_t pX;

	if (num < 0) {
		neg = 1;
		num *= -1;
	}
	do { str[i++] = num % 10 + '0'; } while ((num /= 10) > 0);
	if (neg) str[i++] = '-';
	intLen = i;
	pX = X + (intLen - 1) * (Font->font_Width + 1);
	for (i = 0; i < intLen; i++) {
		pX -= PutChar(pX,Y,str[i],Font, Font_Charset);
	}

    return intLen * (Font->font_Width + 1);
}

// Draw unsigned integer value
// input:
//   X,Y - top left coordinates of first symbol
//   num - unsigned integer value
//   Font - pointer to font
// return: number width in pixels
uint8_t UC1701::PutIntU(uint8_t X, uint8_t Y, uint32_t num, const Font_TypeDef *Font, unsigned char const* Font_Charset) {
	uint8_t str[11]; // 10 chars max for UINT32_MAX
	int8_t i = 0;
	int8_t intLen;
	uint8_t pX;

	do { str[i++] = num % 10 + '0'; } while ((num /= 10) > 0);
	intLen = i;
	pX = X + (intLen - 1) * (Font->font_Width + 1);
	for (i = 0; i < intLen; i++) {
		pX -= PutChar(pX,Y,str[i],Font, Font_Charset);
	}

    return intLen * (Font->font_Width + 1);
}

// Draw signed integer value with decimal point
// input:
//   X,Y - top left coordinates of first symbol
//   num - unsigned integer value
//   decimals - number of digits after decimal point
//   Font - pointer to font
// return: number width in pixels
uint8_t UC1701::PutIntF(uint8_t X, uint8_t Y, int32_t num, uint8_t decimals, const Font_TypeDef *Font, unsigned char const* Font_Charset) {
	uint8_t str[12];
	int8_t i;
	uint8_t neg;
	int8_t strLen;

	if (num < 0) {
		neg  = 1;
		num *= -1;
	} else neg = 0;

	i = 0;
	do { str[i++] = num % 10 + '0'; } while ((num /= 10) > 0);
	strLen = i;
	if (strLen <= decimals) {
		for (i = strLen; i <= decimals; i++) str[i] = '0';
		strLen = decimals + 1;
	}
	if (neg) str[strLen++] = '-';

	neg = X;
	for (i = 0; i < strLen; i++) {
		PutChar(neg,Y,str[strLen - i - 1],Font, Font_Charset);
		neg += Font->font_Width + 1;
		if (strLen - i - 1 == decimals && decimals != 0) {
			Rect(neg,Y + Font->font_Height - 2,neg + 1,Y + Font->font_Height - 1,PSet);
			neg += 3;
		}
	}

	return (neg - X);
}

// Draw signed integer value with leading zeros
// input:
//   X,Y - top left coordinates of first symbol
//   num - unsigned integer value
//   digits - minimal number of length (e.g. num=35, digits=5 --> 00035)
//   Font - pointer to font
// return: number width in pixels
uint8_t UC1701::PutIntLZ(uint8_t X, uint8_t Y, int32_t num, uint8_t digits, const Font_TypeDef *Font, unsigned char const* Font_Charset) {
	uint8_t i;
	uint8_t neg;
	int32_t tmp = num;
	uint8_t len = 0;
	uint8_t pX = X;

	if (tmp < 0) {
		neg  = 1;
		tmp *= -1;
	} else neg = 0;

	do { len++; } while ((tmp /= 10) > 0); // Calculate number length in symbols

	if (len > digits) {
		X += PutInt(X,Y,num,Font, Font_Charset);
		return X;
	}

	if (neg) {
		PutChar(X,Y,'-',Font, Font_Charset);
		X += Font->font_Width + 1;
		num *= -1;
	}
	for (i = 0; i < digits - len; i++) {
		PutChar(X,Y,'0',Font, Font_Charset);
		X += Font->font_Width + 1;
	}
	X += PutInt(X,Y,num,Font, Font_Charset);

	return X - pX;
}

// Draw integer as hexadecimal
// input:
//   X,Y - top left coordinates of first symbol
//   num - unsigned integer value
//   Font - pointer to font
// return: number width in pixels
uint8_t UC1701::PutHex(uint8_t X, uint8_t Y, uint32_t num, const Font_TypeDef *Font, unsigned char const* Font_Charset) {
	uint8_t str[11]; // 10 chars max for UINT32_MAX
	int8_t i = 0;
	uint32_t onum = num;
	int8_t intLen;
	uint8_t pX;

	do { str[i++] = "0123456789ABCDEF"[num % 0x10]; } while ((num /= 0x10) > 0);
	if (onum < 0x10) str[i++] = '0';

	intLen = i;

	pX = X + (intLen - 1) * (Font->font_Width + 1);
	for (i = 0; i < intLen; i++) {
		pX -= PutChar(pX,Y,str[i],Font, Font_Charset);
	}

    return intLen * (Font->font_Width + 1);
}
*/



















