/**
@file UC1701.h

@brief Header file containing member functions and variables

*/

#ifndef __UC1701_H__
#define __UC1701_H__

#include "mbed.h"

/** 
@brief Simple library for interfacing with UC1701

@brief Revision 1.0

@author Neil Hao
@date   January 2016
 *
 * Example:
 * @code
 
#include "mbed.h"
#include "UC1701.h"
#include "font5x7.h"

//init screen, SPI speed is 1Mhz
UC1701 screen(1000000, PB_15, PB_14, PB_13, PA_5, PA_6, PA_7,PC_4);

int main() {
	screen.UC1701_Init();
	screen.UC1701_Contrast(5,32);
	screen.UC1701_Orientation(scr_normal);
	screen.UC1701_Fill(0x00); //clear screen

	screen.PutStr(0, 2, "Challenge", &Font5x7_Def, Font5x7);
	screen.PutStr(35, 14, "Convention", &Font5x7_Def, Font5x7);
	screen.PutStr(0, 26, "Change our", &Font5x7_Def, Font5x7);
	screen.PutStr(60, 38, "World!", &Font5x7_Def, Font5x7);
	screen.UC1701_Flush(); // Send vRAM buffer content into display
		
	while(1){
		;
	}		
}
 
 * @endcode
 */
 
 // Screen dimensions
#define  SCR_W 96
#define  SCR_H 48
 
 		// Public structures
		typedef enum {ON = 0, OFF = !ON} OnOffStatus;
		typedef enum {NORMAL = 0, INVERT = !NORMAL} InvertStatus;
		typedef enum {ENABLED = 0, DISABLED = !ENABLED} DisplayState;
		
		typedef enum {
			scr_normal = 0,
			scr_CW     = 1,
			scr_CCW    = 2,
			scr_180    = 3
		} ScrOrientation_TypeDef;
		
		typedef enum {
			PSet = 1,
			PReset = 0
		} PSetReset_TypeDef;
		
		typedef enum {
	font_V     = 0,        // Vertical font scan lines
	font_H     = 1         // Horizontal font scan lines
} FontScan_TypeDef;

typedef struct {
	uint8_t font_Width;                // Width of character
	uint8_t font_Height;               // Height of character
	uint8_t font_BPC;                  // Bytes for one character
	FontScan_TypeDef font_Scan;        // Font scan lines behavior
} Font_TypeDef;

class UC1701
{
private:
ScrOrientation_TypeDef    scr_orientation;	
uint8_t vRAM[SCR_W * SCR_H / 8]; // Display buffer

	//SPI interface
	SPI * _SPI;

	//Control lines
	DigitalOut * _BL;
	DigitalOut * _A0;
	DigitalOut * _RST;
	DigitalOut * _CS;


	void RTOS_Delay_ms(int ms);
	void UC1701_cmd(uint8_t cmd);
	void UC1701_cmd_double(uint8_t cmd1, uint8_t cmd2);
	void UC1701_data(uint8_t data);


public:
	// Public variables
uint16_t scr_width;
uint16_t scr_height;

    /** Create a UC1701 object connected to the specified pins, with UC1701 back light design, UC1701_BL
    *
    * @param SPI_Hz  SPI Bus Speed
    * @param UC1701_MOSI Pin connected to MOSI
		* @param UC1701_MISO Pin connected to MISO
		* @param UC1701_SCK Pin connected to SCK
		* @param UC1701_CS Pin connected to CS
		* @param UC1701_RST Pin connected to RST
		* @param UC1701_A0 Pin connected to A0
		* @param UC1701_BL Pin connected to BL
    * 
    */
    UC1701(int SPI_Hz, PinName UC1701_MOSI, PinName UC1701_MISO, PinName UC1701_SCK, PinName UC1701_CS, PinName UC1701_RST, PinName UC1701_A0,PinName UC1701_BL);

    /** Create a UC1701 object connected to the specified pins, without UC1701 back light design
    *
    * @param SPI_Hz  SPI Bus Speed
    * @param UC1701_MOSI Pin connected to MOSI
		* @param UC1701_MISO Pin connected to MISO
		* @param UC1701_SCK Pin connected to SCK
		* @param UC1701_CS Pin connected to CS
		* @param UC1701_RST Pin connected to RST
		* @param UC1701_A0 Pin connected to A0
    * 
    */
    UC1701(int SPI_Hz, PinName UC1701_MOSI, PinName UC1701_MISO, PinName UC1701_SCK, PinName UC1701_CS, PinName UC1701_RST, PinName UC1701_A0);
    
		void UC1701_Init(void);
		void UC1701_Reset(void);
		void UC1701_Contrast(uint8_t res_ratio, uint8_t el_vol);
		void UC1701_SetAllPixelOn(OnOffStatus state);
		void UC1701_SetInvert(InvertStatus state);
		void UC1701_SetDisplayState(DisplayState state);
		void UC1701_SetXDir(InvertStatus MX);
		void UC1701_SetYDir(InvertStatus MY);
		void UC1701_SetAddr(uint8_t X, uint8_t Y);
		void UC1701_SetScrollLine(uint8_t line);
		void UC1701_Orientation(uint8_t orientation);
		void UC1701_Flush(void);
		void UC1701_Fill(uint8_t pattern);
		void SetPixel(uint8_t X, uint8_t Y);
		void ResetPixel(uint8_t X, uint8_t Y);
		void InvertRect(uint8_t X, uint8_t Y, uint8_t W, uint8_t H);
		void HLine(uint8_t X1, uint8_t X2, uint8_t Y, PSetReset_TypeDef SR);
		void VLine(uint8_t X, uint8_t Y1, uint8_t Y2, PSetReset_TypeDef SR);
		void Rect(uint8_t X1, uint8_t Y1, uint8_t X2, uint8_t Y2, PSetReset_TypeDef SR);
		void FillRect(uint8_t X1, uint8_t Y1, uint8_t X2, uint8_t Y2, PSetReset_TypeDef SR);
		void Line(int16_t X1, int16_t Y1, int16_t X2, int16_t Y2);
		void Ellipse(uint16_t X, uint16_t Y, uint16_t A, uint16_t B);
		uint8_t PutChar(uint8_t X, uint8_t Y, uint8_t Char, const Font_TypeDef *Font, unsigned char const* Font_Charset);
		uint16_t PutStr(uint8_t X, uint8_t Y, char *str, const Font_TypeDef *Font, unsigned char const* Font_Charset);
		
		/*
		uint16_t PutStrLF(uint8_t X, uint8_t Y, char *str, const Font_TypeDef *Font, unsigned char const* Font_Charset);
		uint8_t PutInt(uint8_t X, uint8_t Y, int32_t num, const Font_TypeDef *Font, unsigned char const* Font_Charset);
		uint8_t PutIntU(uint8_t X, uint8_t Y, uint32_t num, const Font_TypeDef *Font, unsigned char const* Font_Charset);
		uint8_t PutIntF(uint8_t X, uint8_t Y, int32_t num, uint8_t decimals, const Font_TypeDef *Font, unsigned char const* Font_Charset);
		uint8_t PutIntLZ(uint8_t X, uint8_t Y, int32_t num, uint8_t digits, const Font_TypeDef *Font, unsigned char const* Font_Charset);
		uint8_t PutHex(uint8_t X, uint8_t Y, uint32_t num, const Font_TypeDef *Font, unsigned char const* Font_Charset);
		*/

};


#endif /* __UC1701_H__ */
